import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


from sklearn.linear_model import LogisticRegression


from chembee_config.benchmark.BenchmarkAlgorithm import BenchmarkAlgorithm


class LogisticRegressionClassifier(BenchmarkAlgorithm):
    name = "logistic-regression"

    algorithms = (
        LogisticRegression(penalty="l1", solver="saga", max_iter=1000),
        LogisticRegression(penalty="l2", solver="saga", max_iter=1000),
        LogisticRegression(
            penalty="elasticnet", solver="saga", l1_ratio=0.5, max_iter=1000
        ),
    )

    titles = (
        "l1",
        "l2",
        "elasticnet with $\chi(\mathrm{L!}) = 0.5$",
    )



import sys, os
from sklearn import datasets

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from utils import prepare_data_decicion_lib
from chembee_actions.benchmark_algorithms import benchmark_standard
from chembee_actions.cross_validation import screen_cross_validation_grid_search

from chembee_config.calibration.random_forest import RandomForestClassifier
from chembee_config.calibration.knn import KNNClassifier
from chembee_config.benchmark.grid_search_cv import GridSearchCVClassifier

from chembee_datasets.IrisDataSet import IrisDataSet


def test_benchmarking():

    # Data Definition
    DataSet = IrisDataSet(split_ratio=0.8)
    # Grid search
    scores = ["precision_macro", "recall_macro", "accuracy"]
    names = ["rf", "knn"]
    clf_list = [KNNClassifier, RandomForestClassifier]

    fitted_clf, result_clf = screen_cross_validation_grid_search(
        scores=scores,
        clf_list=clf_list,
        X_train=DataSet.X_train,
        X_test=DataSet.X_test,
        y_train=DataSet.y_train,
        y_test=DataSet.y_test,
        refit="accuracy",
    )

    GridSearchCV = GridSearchCVClassifier(clf_list, names)
    iris = datasets.load_iris()
    X, y = prepare_data_decicion_lib(iris)
    benchmark_standard(
        X,
        y,
        feature_names=iris.feature_names[:2],
        algorithms=[GridSearchCV],
        to_fit=False,
    )

    # Standard screen

    benchmark_standard(
        X,
        y,
        feature_names=iris.feature_names[:2],
    )



import os
import sys


sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


from sklearn.linear_model import LinearRegression, Ridge, Lasso, ElasticNet


from chembee_config.benchmark.BenchmarkAlgorithm import BenchmarkAlgorithm


class LinearRegressionClassifier(BenchmarkAlgorithm):

    name = "linear-regression"

    algorithms = (LinearRegression(), Ridge(), Lasso(), ElasticNet())
    titles = (
        "Ordinary Least Squares",
        "Ridge",
        "Lasso",
        "Elastic net",
    )



import matplotlib.pyplot as plt
import pandas as pd
from file_utils import insert_string_in_file_name, prepare_file_name_saving


def plot_feature_importances(result_json: dict, file_name: str, prefix: str):
    """
    The plot_feature_importances function plots the feature importances of a random forest model.
    It takes three arguments:
        - forest_importances: The feature importances from the random forest model, as returned by sklearn's .feature_importance() method.
        - std: The standard deviation of the feature importances, as returned by sklearn's .std() method on a RandomForestRegressor or RandomForestClassifier object.  This is used to plot error bars for each importance value.  If no standard deviation is available (i.e., if you are using a DecisionTreeRegressor or DecisionTreeClassifier), set this to None (the default).
        - file_name: A string containing the name of your desired output file (e.g., "FeatureImportancePlot" will result in "FeatureImportancePlotRFModelNameHere").

    :param forest_importances: Used to pass the feature importances to the function.
    :param std: Used to plot the standard deviation of the feature importances.
    :param file_name:std: Used to pass the file name of the plot to be saved.
    :param prefix:std: Used to specify the prefix for the file name.
    :return: The feature importances of the forest.

    :doc-author: Trelent
    """
    importances = result_json["importances"]
    feature_indices = result_json["feature_indices"]
    std = result_json["std"]
    forest_importances = pd.Series(importances, index=feature_indices)
    file_name = insert_string_in_file_name(
        file_name, insertion="FeatureImportancesRF", ending=".png"
    )
    file_name = prepare_file_name_saving(
        prefix=prefix, file_name=file_name, ending=".png"
    )
    fig, ax = plt.subplots()
    forest_importances.plot.bar(yerr=std, ax=ax)
    ax.set_xlabel("Feature No.")
    ax.set_ylabel("Mean decrease in impurity")
    fig.tight_layout()
    fig.savefig(file_name, dpi=300)



import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from sklearn.neural_network import MLPClassifier

from chembee_config.benchmark.BenchmarkAlgorithm import BenchmarkAlgorithm


class NeuralNetworkClassifier(BenchmarkAlgorithm):

    name = "multilayer-perceptron"
    hidden_layer_sizes = (100, 20, 20, 100)
    max_iter = 10000
    algorithms = (
        MLPClassifier(
            hidden_layer_sizes=hidden_layer_sizes,
            activation="logistic",
            max_iter=max_iter,
        ),
        MLPClassifier(
            hidden_layer_sizes=hidden_layer_sizes, activation="tanh", max_iter=max_iter
        ),
        MLPClassifier(
            hidden_layer_sizes=hidden_layer_sizes, activation="relu", max_iter=max_iter
        ),
        MLPClassifier(
            hidden_layer_sizes=hidden_layer_sizes,
            activation="relu",
            solver="adam",
            max_iter=max_iter,
        ),
    )
    titles = (
        "Logistic with SGD",
        "Tanh with SGD",
        "RELU with SGD",
        "RELU with Adam",
    )



import sys, os

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from chembee_datasets.BreastCancer import BreastCancerDataset
from chembee_actions.evaluation import screen_classifier_for_metrics
from chembee_plotting.evaluation import plot_collection

from chembee_config.calibration.random_forest import RandomForestClassifier
from chembee_config.calibration.knn import KNNClassifier
from chembee_config.calibration.mlp_classifier import NeuralNetworkClassifierRELU
from chembee_config.calibration.svc import NaivelyCalibratedSVCRBF


def test_screen_classifier_for_metrics():

    DataSet = BreastCancerDataset(split_ratio=0.8)
    metrics = screen_classifier_for_metrics(
        X_train=DataSet.X_train,
        X_test=DataSet.X_test,
        y_train=DataSet.y_train,
        y_test=DataSet.y_test,
    )
    assert ["scalar", "array", "matrix"] == list(metrics.keys())
    plot_collection(
        metrics, file_name=DataSet.name + "_evaluation", prefix="plots/evaluation"
    )
    # Test for fitted classifiers
    clf_list = [
        RandomForestClassifier,
        KNNClassifier,
        NeuralNetworkClassifierRELU,
        NaivelyCalibratedSVCRBF,
    ]
    clf_result = []
    for clf in clf_list:
        clf.fit(DataSet.X_train, DataSet.y_train)
        clf_result.append(clf)
    metrics = screen_classifier_for_metrics(
        X_train=DataSet.X_train,
        X_test=DataSet.X_test,
        y_train=DataSet.y_train,
        y_test=DataSet.y_test,
        clf_list=clf_list,
    )
    assert ["scalar", "array", "matrix"] == list(metrics.keys())



from sklearn.neural_network import BernoulliRBM

import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


RBMClassifier = BernoulliRBM(n_components=612, learning_rate=0.0001)
